@@ -6,7 +6,7 @@ from cf_remote import log
 from cf_remote import version
 from cf_remote import commands, paths
 from cf_remote.utils import user_error, exit_success, expand_list_from_file, is_file_string
-from cf_remote.utils import strip_user, read_json, is_package_url
+from cf_remote.utils import strip_user, read_json, is_package_url, cache
 from cf_remote.packages import Releases
 from cf_remote.spawn import Providers
 
@@ -17,8 +17,8 @@ def print_version_info():
     releases = Releases()
     print(releases)
 
-
-def get_args():
+@cache
+def _get_arg_parser():
     ap = argparse.ArgumentParser(
         description="Spooky CFEngine at a distance",
         formatter_class=argparse.ArgumentDefaultsHelpFormatter)
@@ -76,7 +76,7 @@ def get_args():
     sp.add_argument("--raw", help="Print only output of command itself", action='store_true')
     sp.add_argument("remote_command", help="Command to execute on remote host (including args)", type=str, nargs=1)
 
-    sp = subp.add_parser("save", help="Spawn hosts in the clouds")
+    sp = subp.add_parser("save", help="Save host(s) with a group name to use in other commands")
     sp.add_argument("--role", help="Role of the hosts", choices=["hub", "hubs", "client", "clients"], required=True)
     sp.add_argument("--name", help="Name of the group of hosts (can be used in other commands)", required=True)
     sp.add_argument("--hosts", "-H", help="SSH usernames and IPs for SSH and CFEngine in the form of user@ip", required=True)
@@ -118,9 +118,13 @@ def get_args():
     dp.add_argument("name", help="Name fo the group of hosts to destroy", nargs='?')
 
     sp = subp.add_parser("deploy", help="Deploy masterfiles to hub")
-    sp.add_argument("--hub", help="Hub(s) to deploy to", type=str, required=True)
-    sp.add_argument("masterfiles", help="Path to local masterfiles directory or tarball", type=str)
+    sp.add_argument("--hub", help="Hub(s) to deploy to", type=str)
+    sp.add_argument("masterfiles", help="Path to local masterfiles directory or tarball", type=str, nargs="?")
 
+    return ap
+
+def get_args():
+    ap = _get_arg_parser()
     args = ap.parse_args()
     return args
 
@@ -259,16 +263,13 @@ def validate_command(command, args):
         if not args.all and not args.name:
             user_error("One of --all or NAME required for destroy")
 
-    if command == "deploy":
-        if not args.masterfiles:
-            user_error("Must specify a path to masterfiles")
-        if not args.hub:
-            user_error("Must specify at least one hub")
-        if args.masterfiles.startswith(("http://", "https://")):
-            if not args.masterfiles.endswith((".tgz", ".tar.gz")):
+    if command == "deploy" and args.masterfiles:
+        masterfiles = args.masterfiles
+        if masterfiles.startswith(("http://", "https://")):
+            if not masterfiles.endswith((".tgz", ".tar.gz")):
                 user_error("masterfiles URL must be to a gzipped tarball (.tgz or .tar.gz)")
-        elif not os.path.exists(args.masterfiles):
-            user_error(f"'{args.masterfiles}' does not exist")
+        elif not os.path.exists(masterfiles):
+            user_error(f"'{masterfiles}' does not exist")
 
 
 def is_in_cloud_state(name):
@@ -383,6 +384,7 @@ def validate_args(args):
         args.hub = resolve_hosts(args.hub)
 
     if not args.command:
+        _get_arg_parser().print_help()
         user_error("Invalid or missing command")
     args.command = args.command.strip()
     validate_command(args.command, args)
